#include "chibi.h"
#include <limits.h>

#define COLLECTION_PERIOD 30000
#define SENSOR_INPUT_PIN 2

unsigned long pulseStart, pulseDuration, now;
float ratio, concentration;

void setup()
{
  chibiCmdInit(57600);
  attachInterrupt(0, isr, CHANGE);
  pulseDuration = 0;
  now = millis();
}

void loop()
{
  chibiCmdPoll();
  if (elapsedTime(now) > COLLECTION_PERIOD)
  {
    ratio = (float)pulseDuration/float(COLLECTION_PERIOD);
    
    // calculate an integer percentage
    ratio *= 100.0;
    
    // using formula calculated by Christopher Nafis
    // http://www.howmuchsnow.com/arduino/airquality/grovedust/
    // note: I removed the constant zero offset value (0.62) from the formula cuz it's weird
    // to have a ratio of 0 and then a positive particulate concentration. even though its not possible
    // to have a concentration of 0 unless it's a vacuum or something. its just weird.
    concentration = 1.1*pow(ratio,3)-3.8*pow(ratio,2)+520*ratio;
    
    Serial.println(concentration);
    //Serial.print("Ratio: ");Serial.print(ratio); Serial.print(" , Concentration: "); Serial.println(concentration);
    
    now = millis();
    pulseDuration = 0;
  }
}

/**************************************************************************/
// calculate length of pulses
/**************************************************************************/
void isr()
{
  // check to see if we're starting a particle capture or finishing one
  if (digitalRead(SENSOR_INPUT_PIN) == 0)
  {
    //Serial.println("LOW");
    // if the signal just transitioned low, then log the timestamp
    pulseStart = millis();
  }
  else
  {
    //Serial.println("HIGH");
    // if the signal just transitioned high, calculate the elapsed time and add it to the total duration
    pulseDuration += elapsedTime(pulseStart);
  }
}

/**************************************************************************/
// calculate elapsed time. this takes into account rollover.
/**************************************************************************/
unsigned long elapsedTime(unsigned long startTime)
{
  unsigned long stopTime = millis();
  
  if (stopTime >= startTime)
  {
    return stopTime - startTime;
  }
  else
  {
    return (ULONG_MAX - (startTime - stopTime));
  }
}

